@@ -1,4 +1,4 @@
-from typing import List, Optional, Tuple, Dict, Callable, Any, Union
+from typing import List, Optional, Tuple, Union
 from functools import reduce
 import os
 import sys
@@ -6,7 +6,7 @@ import asyncio
 from pathlib import Path
 import re
 from .paths import CACHE_PATH
-from .utils import maybe_is_text, maybe_is_truncated
+from .utils import maybe_is_text, md5sum
 from .qaprompts import (
     summary_prompt,
     qa_prompt,
@@ -15,14 +15,13 @@ from .qaprompts import (
     select_paper_prompt,
     make_chain,
 )
-from dataclasses import dataclass
+from .types import Answer, Context
 from .readers import read_doc
 from langchain.vectorstores import FAISS
 from langchain.embeddings.openai import OpenAIEmbeddings
 from langchain.embeddings.base import Embeddings
 from langchain.chat_models import ChatOpenAI
 from langchain.llms.base import LLM
-from langchain.chains import LLMChain
 from langchain.callbacks import get_openai_callback
 from langchain.cache import SQLiteCache
 import langchain
@@ -32,32 +31,6 @@ os.makedirs(os.path.dirname(CACHE_PATH), exist_ok=True)
 langchain.llm_cache = SQLiteCache(CACHE_PATH)
 
 
-@dataclass
-class Answer:
-    """A class to hold the answer to a question."""
-
-    question: str
-    answer: str = ""
-    context: str = ""
-    contexts: List[Any] = None
-    references: str = ""
-    formatted_answer: str = ""
-    passages: Dict[str, str] = None
-    tokens: int = 0
-    cost: float = 0
-
-    def __post_init__(self):
-        """Initialize the answer."""
-        if self.contexts is None:
-            self.contexts = []
-        if self.passages is None:
-            self.passages = {}
-
-    def __str__(self) -> str:
-        """Return the answer as a string."""
-        return self.formatted_answer
-
-
 class Docs:
     """A collection of documents to be used for answering questions."""
 
@@ -72,8 +45,6 @@ class Docs:
     ) -> None:
         """Initialize the collection of documents.
 
-
-
         Args:
             chunk_size_limit: The maximum number of characters to use for a single chunk of text.
             llm: The language model to use for answering questions. Default - OpenAI chat-gpt-turbo
@@ -123,18 +94,20 @@ class Docs:
         key: Optional[str] = None,
         disable_check: bool = False,
         chunk_chars: Optional[int] = 3000,
+        overwrite: bool = False,
     ) -> None:
         """Add a document to the collection."""
 
         # first check to see if we already have this document
         # this way we don't make api call to create citation on file we already have
+        md5 = md5sum(path)
         if path in self.docs:
             raise ValueError(f"Document {path} already in collection.")
 
         if citation is None:
             # peak first chunk
             texts, _ = read_doc(path, "", "", chunk_chars=chunk_chars)
-            with get_openai_callback() as cb:
+            with get_openai_callback():
                 citation = self.cite_chain.run(texts[0])
             if len(citation) < 3 or "Unknown" in citation or "insufficient" in citation:
                 citation = f"Unknown, {os.path.basename(path)}, {datetime.now().year}"
@@ -161,7 +134,6 @@ class Docs:
             else:
                 suffix = chr(ord(suffix) + 1)
         key += suffix
-        self.keys.add(key)
 
         texts, metadata = read_doc(path, citation, key, chunk_chars=chunk_chars)
         # loose check to see if document was loaded
@@ -172,12 +144,12 @@ class Docs:
             raise ValueError(
                 f"This does not look like a text document: {path}. Path disable_check to ignore this error."
             )
-
-        self.docs[path] = dict(texts=texts, metadata=metadata, key=key)
         if self._faiss_index is not None:
             self._faiss_index.add_texts(texts, metadatas=metadata)
         if self._doc_index is not None:
             self._doc_index.add_texts([citation], metadatas=[{"key": key}])
+        self.docs[path] = dict(texts=texts, metadata=metadata, key=key, md5=md5)
+        self.keys.add(key)
 
     def clear(self) -> None:
         """Clear the collection of documents."""
@@ -207,7 +179,7 @@ class Docs:
     def doc_match(self, query: str, k: int = 25) -> List[str]:
         """Return a list of documents that match the query."""
         if len(self.docs) == 0:
-            return ''
+            return ""
         if self._doc_index is None:
             texts = [doc["metadata"][0]["citation"] for doc in self.docs.values()]
             metadatas = [
@@ -246,6 +218,8 @@ class Docs:
         except:
             # they use some special exception type, but I don't want to import it
             self._faiss_index = None
+        if not hasattr(self, "_doc_index"):
+            self._doc_index = None
         self.update_llm(None, None)
 
     def _build_faiss_index(self):
@@ -267,10 +241,11 @@ class Docs:
         max_sources: int = 5,
         marginal_relevance: bool = True,
         key_filter: Optional[List[str]] = None,
-    ) -> str:
+    ) -> Answer:
         # special case for jupyter notebooks
         if "get_ipython" in globals() or "google.colab" in sys.modules:
             import nest_asyncio
+
             nest_asyncio.apply()
         try:
             loop = asyncio.get_event_loop()
@@ -294,7 +269,7 @@ class Docs:
         max_sources: int = 5,
         marginal_relevance: bool = True,
         key_filter: Optional[List[str]] = None,
-    ) -> str:
+    ) -> Answer:
         if len(self.docs) == 0:
             return answer
         if self._faiss_index is None:
@@ -316,19 +291,19 @@ class Docs:
             if key_filter is not None and doc.metadata["dockey"] not in key_filter:
                 return None
             # check if it is already in answer (possible in agent setting)
-            if doc.metadata["key"] in [c[0] for c in answer.contexts]:
+            if doc.metadata["key"] in [c.key for c in answer.contexts]:
                 return None
-            c = (
-                doc.metadata["key"],
-                doc.metadata["citation"],
-                await self.summary_chain.arun(
+            c = Context(
+                key=doc.metadata["key"],
+                citation=doc.metadata["citation"],
+                context=await self.summary_chain.arun(
                     question=answer.question,
                     context_str=doc.page_content,
                     citation=doc.metadata["citation"],
                 ),
-                doc.page_content,
+                text=doc.page_content,
             )
-            if "Not applicable" not in c[2]:
+            if "Not applicable" not in c.context:
                 return c
             return None
 
@@ -339,18 +314,24 @@ class Docs:
         contexts = [c for c in contexts if c is not None]
         if len(contexts) == 0:
             return answer
-        contexts = sorted(contexts, key=lambda x: x[2], reverse=True)
+        contexts = sorted(contexts, key=lambda x: len(x.context), reverse=True)
         contexts = contexts[:max_sources]
         # add to answer (if not already there)
-        keys = [c[0] for c in answer.contexts]
+        keys = [c.key for c in answer.contexts]
         for c in contexts:
-            if c[0] not in keys:
+            if c.key not in keys:
                 answer.contexts.append(c)
 
         context_str = "\n\n".join(
-            [f"{k}: {s}" for k, c, s, t in answer.contexts if "Not applicable" not in s]
+            [
+                f"{c.key}: {c.context}"
+                for c in answer.contexts
+                if "Not applicable" not in c.context
+            ]
         )
-        valid_keys = [k for k, c, s, t in answer.contexts if "Not applicable" not in s]
+        valid_keys = [
+            c.key for c in answer.contexts if "Not applicable" not in c.context
+        ]
         if len(valid_keys) > 0:
             context_str += "\n\nValid keys: " + ", ".join(valid_keys)
         answer.context = context_str
@@ -379,7 +360,7 @@ class Docs:
         marginal_relevance: bool = True,
         answer: Optional[Answer] = None,
         key_filter: Optional[bool] = None,
-    ):
+    ) -> Answer:
         # special case for jupyter notebooks
         if "get_ipython" in globals() or "google.colab" in sys.modules:
             import nest_asyncio
@@ -398,7 +379,7 @@ class Docs:
                 length_prompt=length_prompt,
                 marginal_relevance=marginal_relevance,
                 answer=answer,
-                key_filter=key_filter
+                key_filter=key_filter,
             )
         )
 
@@ -410,13 +391,13 @@ class Docs:
         length_prompt: str = "about 100 words",
         marginal_relevance: bool = True,
         answer: Optional[Answer] = None,
-        key_filter: Optional[bool] = None
-    ):
+        key_filter: Optional[bool] = None,
+    ) -> Answer:
         if k < max_sources:
             raise ValueError("k should be greater than max_sources")
         if answer is None:
             answer = Answer(query)
-        if key_filter or (key_filter is None and len(self.docs) > 5)    :
+        if key_filter or (key_filter is None and len(self.docs) > 5):
             with get_openai_callback() as cb:
                 keys = self.doc_match(answer.question)
             answer.tokens += cb.total_tokens
@@ -427,9 +408,9 @@ class Docs:
                 k=k,
                 max_sources=max_sources,
                 marginal_relevance=marginal_relevance,
-                key_filter=keys if key_filter else None
+                key_filter=keys if key_filter else None,
             )
-        context_str, citations = answer.context, answer.contexts
+        context_str, contexts = answer.context, answer.contexts
         bib = dict()
         passages = dict()
         if len(context_str) < 10:
@@ -446,7 +427,10 @@ class Docs:
         # it still happens lol
         if "(Foo2012)" in answer_text:
             answer_text = answer_text.replace("(Foo2012)", "")
-        for key, citation, summary, text in citations:
+        for c in contexts:
+            key = c.key
+            text = c.context
+            citation = c.citation
             # do check for whole key (so we don't catch Callahan2019a with Callahan2019)
             skey = key.split(" ")[0]
             if skey + " " in answer_text or skey + ")" or skey + "," in answer_text:
