@@ -45,6 +45,7 @@ def test_evidence():
     evidence = docs.get_evidence(
         paperqa.Answer("For which state was he a governor"), k=1, max_sources=1
     )
+    print(evidence.contexts[0].context, evidence.context)
     assert "Missouri" in evidence.context
     os.remove(doc_path)
 
@@ -57,7 +58,7 @@ def test_query():
         f.write(r.text)
     docs = paperqa.Docs()
     docs.add(doc_path, "WikiMedia Foundation, 2023, Accessed now")
-    answer = docs.query("What is Frederick Bates's greatest accomplishment?")
+    docs.query("What is Frederick Bates's greatest accomplishment?")
     os.remove(doc_path)
 
 
@@ -72,8 +73,7 @@ class Test(IsolatedAsyncioTestCase):
             f.write(r.text)
         docs = paperqa.Docs()
         docs.add(doc_path, "WikiMedia Foundation, 2023, Accessed now")
-        i = 0
-        answer = await docs.aquery("What is Frederick Bates's greatest accomplishment?")
+        await docs.aquery("What is Frederick Bates's greatest accomplishment?")
         os.remove(doc_path)
 
 
@@ -85,8 +85,7 @@ def test_doc_match():
         f.write(r.text)
     docs = paperqa.Docs()
     docs.add(doc_path, "WikiMedia Foundation, 2023, Accessed now")
-    i = 0
-    result = docs.doc_match("What is Frederick Bates's greatest accomplishment?")
+    docs.doc_match("What is Frederick Bates's greatest accomplishment?")
     os.remove(doc_path)
 
 
@@ -204,7 +203,7 @@ def test_code():
     docs = paperqa.Docs(llm=OpenAI(temperature=0.0, model_name="text-ada-001"))
     docs.add(doc_path, "test_paperqa.py", key="test", disable_check=True)
     assert len(docs.docs) == 1
-    answer = docs.query("What function tests the preview?")
+    docs.query("What function tests the preview?")
 
 
 def test_citation():
@@ -244,9 +243,17 @@ def test_query_filter():
         r = requests.get("https://en.wikipedia.org/wiki/Frederick_Bates_(politician)")
         f.write(r.text)
     docs = paperqa.Docs()
-    docs.add(doc_path, "Information about Fredrick Bates, WikiMedia Foundation, 2023, Accessed now")
+    docs.add(
+        doc_path,
+        "Information about Fredrick Bates, WikiMedia Foundation, 2023, Accessed now",
+    )
     # add with new dockey
     docs.add("example.txt", "WikiMedia Foundation, 2023, Accessed now", key="test")
     answer = docs.query("What country is Bates from?", key_filter=True)
-    print(answer.context)
-    assert "United States" in answer.answer
\ No newline at end of file
+    # the filter shouldn't trigger, so just checking that it doesn't crash
+
+
+def disabled_test_agent():
+    docs = paperqa.Docs()
+    answer = paperqa.run_agent(docs, "What compounds target AKT1")
+    print(answer)
